#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Ben Jones'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from future import standard_library
standard_library.install_aliases()

import urllib.request, urllib.parse, urllib.error
import base64

try:
    import simplejson as json
except ImportError:
    import json


def plugin(srv, item):
    """ addrs: (method, url, dict(params), list(username, password), json) """

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    method = item.addrs[0]
    url    = item.addrs[1]
    params = item.addrs[2]
    timeout = item.config.get('timeout', 60)

    auth = None
    try:
        username, password = item.addrs[3]
        auth = base64.encodestring('%s:%s' % (username, password)).replace('\n', '')
    except:
        pass

    tojson = None
    try:
        tojson = item.addrs[4]
    except:
        pass

    # Try and transform the URL. Use original URL if it's not possible
    try:
        url = url.format(**item.data)
    except:
        pass

    if params is not None:
        for key in list(params.keys()):

            # { 'q' : '@message' }
            # Quoted field, starts with '@'. Do not use .format, instead grab
            # the item's [message] and inject as parameter value.
            if params[key].startswith('@'):         # "@message"
                params[key] = item.get(params[key][1:], "NOP")

            else:
                try:
                    params[key] = params[key].format(**item.data).encode('utf-8')
                except Exception as e:
                    srv.logging.exception("Parameter %s cannot be formatted" % key)
                    return False

    message  = item.message

    if method.upper() == 'GET':
        try:
            if params is not None:
                resource = url
                if not resource.endswith('?'):
                    resource = resource + '?'
                resource = resource + urllib.parse.urlencode(params)
            else:
                resource = url

            request = urllib.request.Request(resource)

            if srv.SCRIPTNAME is not None:
                request.add_header('User-agent', srv.SCRIPTNAME)
            if auth is not None:
                request.add_header("Authorization", "Basic %s" % auth)

            resp = urllib.request.urlopen(request, timeout=timeout)
            data = resp.read()
            #srv.logging.debug("HTTP response:\n%s" % data)
        except Exception as e:
            srv.logging.warn("Cannot GET %s: %s" % (resource, e))
            return False

        return True

    if method.upper() == 'POST':
        try:
            request = urllib.request.Request(url)
            if params is not None:
                if tojson is not None:
                    encoded_params = json.dumps(params)
                    request.add_header('Content-Type', 'application/json')
                else:
                    encoded_params = urllib.parse.urlencode(params)
            else:
                if tojson is not None:
                    encoded_params = item.payload
                    request.add_header('Content-Type', 'application/json')
                else:
                    encoded_params = message


            request.data = encoded_params.encode('utf-8')

            if srv.SCRIPTNAME is not None:
                request.add_header('User-agent', srv.SCRIPTNAME)
            if auth is not None:
                request.add_header("Authorization", "Basic %s" % auth)

            srv.logging.debug("before send")
            resp = urllib.request.urlopen(request, timeout=timeout)
            data = resp.read()
            #srv.logging.debug("HTTP response:\n%s" % data)
        except Exception as e:
            srv.logging.warn("Cannot POST %s: %s" % (url, e))
            return False

        return True

    srv.logging.warn("Unsupported HTTP method: %s" % (method))
    return False



# -*- coding: utf-8 -*-
# (c) 2018 The mqttwarn developers

# Configuration- and function files used by the test harness
configfile_full = 'tests/etc/full.ini'
configfile_service_loading = 'tests/etc/service-loading.ini'
configfile_no_functions = 'tests/etc/no-functions.ini'
configfile_empty_functions = 'tests/etc/empty-functions.ini'
configfile_unknown_functions = 'tests/etc/unknown-functions.ini'
funcfile_good = 'tests/etc/functions_good.py'
funcfile_bad = 'tests/etc/functions_bad.py'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Artem Alexandrov <qk4l@tem4uk.ru>'
__copyright__ = 'Copyright 2016 Artem Alexandrov'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'


from builtins import str
from builtins import object
import requests
try:
    import simplejson as json
except ImportError:
    import json


def plugin(srv, item):
    """
    addrs: (tg_contact, token, text)
    """
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    token = item.config['token']
    if 'use_chat_id' in item.config:
        useChatId = int(item.config.get('use_chat_id', 0))
    else:
        useChatId = False
    if 'parse_mode' in item.config:
        parse_mode = item.config['parse_mode']
    tg_contact = item.addrs[0]

    class TelegramAPI(object):
        def __init__(self, token, parse_mode=None):
            self.token = token
            self.disable_notification = False
            self.parse_mode = parse_mode
            self.tg_url_bot_general = "https://api.telegram.org/bot"

        def http_get(self, url):
            res = requests.get(url)
            answer = res.text
            answer_json = json.loads(answer)
            return answer_json

        def get_updates(self):
            """
            Get updates
            :return: Dict of last bot update
            """
            url = self.tg_url_bot_general + self.token + "/getUpdates"
            srv.logging.debug(url)
            updates = self.http_get(url)
            #srv.logging.debug("Content of /getUpdates: %s" % updates)
            if not updates["ok"]:
                srv.logging.warn(updates)
                return False
            else:
                return updates

        def get_uid(self, name):
            """
            Get chat_id for specific user
            :param name: First Name or @username of user or #chat_id
            :return: string
            """
            if name.startswith('#'):
                #chat_id was specified in mqttwarn.ini
                return name[1:]
            uid = 0
            srv.logging.debug("Getting uid from /getUpdates...")
            updates = self.get_updates()
            if name.startswith('@'):
                name = name[1:]
                name_type = 'username'
            else:
                name_type = 'first_name'
            for msg in updates["result"]:
                chat = msg["message"]["chat"]
                if name_type in chat:
                    if chat[name_type] == name:
                        uid = chat["id"]
                        break
            srv.logging.debug("For user {name} follow chat_id was found: {chat_id}".format(chat_id=uid,
                                                                                           name=name))
            return uid

        def send_message(self, chat_id, message):
            """
            Send message to chat_id
            :param chat_id: int
            :param message: string
            :return: Boolean
            """
            url = self.tg_url_bot_general + self.token + "/sendMessage"
            params = {"chat_id": chat_id, "text": message,
                      "parse_mode": self.parse_mode, "disable_notification": self.disable_notification}
            srv.logging.debug("Trying to /sendMessage: {url}".format(url=url))
            srv.logging.debug("post params: %r", params)
            res = requests.post(url, params=params)
            answer = res.text
            answer_json = json.loads(answer)
            if not answer_json["ok"]:
                srv.logging.warn(answer_json)
                return False
            else:
                return answer_json

    try:
        tg = TelegramAPI(token, parse_mode)
        if useChatId:
            srv.logging.debug("Setting chatid directly to %r", tg_contact)
            uid = int(tg_contact)
        else:
            uid = tg.get_uid(tg_contact)
        if uid == 0:
            srv.logging.warn("Cannot get chat_id for user %r", tg_contact)
            return False
        reply = tg.send_message(uid, item.message)
        srv.logging.debug("Telegram reply: %r", reply)
        if 'ok' in reply and reply['ok']:
            return True
        return False
    except Exception:
        srv.logging.error("Failed to send request to Telegram", exc_info=True)
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Based on xmpp plugin
__originalauthor__ = 'Fabian Affolter <fabian()affolter-engineering.ch>'
__author__         = 'Remi Vincent <remi.vincent()gmail.com>'
__copyright__      = 'Copyright 2020 Remi Vincent'
__license__        = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import slixmpp
import asyncio

class send_msg_bot(slixmpp.ClientXMPP):
    def __init__(self, sender, password, recipient, message, loop):
        self.loop = loop
        asyncio.set_event_loop(loop)
        slixmpp.ClientXMPP.__init__(self, sender, password)
        self.recipient = recipient
        self.message = message
        self.add_event_handler("session_start", self.start)

    async def start(self, event):
        self.send_message(mto = self.recipient, mbody = self.message, mtype = 'chat')
        self.disconnect()

def plugin(srv, item):
    """Send a message to XMPP recipient(s)."""

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    xmpp_addresses = item.addrs
    sender = item.config['sender']
    password = item.config['password']
    text = item.message

    if not xmpp_addresses:
        srv.logging.warn("Skipped sending XMPP notification to %s, "
                         "no addresses configured" % (item.target))
        return False

    try:
        srv.logging.debug("Sending XMPP notification to %s, addresses: %s" % (item.target, xmpp_addresses))
        loop = asyncio.new_event_loop()
        for target in xmpp_addresses:
            xmpp = send_msg_bot(sender, password, target, text, loop)
            xmpp.connect()
            xmpp.process(forever=False)
        srv.logging.debug("Successfully sent message")
    except Exception as e:
        srv.logging.error("Error sending message to %s: %s" % (item.target, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from future import standard_library
standard_library.install_aliases()
import os
import json
import urllib.request, urllib.parse, urllib.error
import urllib.request, urllib.error, urllib.parse
import urllib.parse

PUSHSAFER_API = "https://www.pushsafer.com/"


class PushsaferError(Exception): pass


def pushsafer(**kwargs):
    assert 'm' in kwargs

    if not kwargs['k']:
        kwargs['k'] = os.environ['PUSHSAFER_TOKEN']

    url = urllib.parse.urljoin(PUSHSAFER_API, "api")
    data = urllib.parse.urlencode(kwargs).encode('utf-8')
    req = urllib.request.Request(url, data)
    response = urllib.request.urlopen(req, timeout=3)
    output = response.read()
    data = json.loads(output)

    if data['status'] != 1:
        raise PushsaferError(output)


def plugin(srv, item):

    message  = item.message
    addrs    = item.addrs
    title    = item.title


    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)
    
    # based on the Pushsafer API (https://www.pushsafer.com/en/pushapi)
    # addrs is an array with two or three elements:
    # 0 is the private or alias key
    # 1 (if present) is the Pushsafer device or device group id where the message is to be sent
    # 2 (if present) is the Pushsafer icon to display in the message
    # 3 (if present) is the Pushsafer sound to play for the message
    # 4 (if present) is the Pushsafer vibration for the message
    # 5 (if present) is the Pushsafer URL or URL Scheme
    # 6 (if present) is the Pushsafer Title of URL
    # 7 (if present) is the Pushsafer Time in minutes, after which message automatically gets purged
    # 8 (if present) is the Pushsafer priority, integer -2, -1, 0, 1, 2
    # 9 (if present) is the Pushsafer retry after which time (in secomds 60-10800) a message should resend
    # 10 (if present) is the Pushsafer expire after which time (in secomds 60-10800) the retry should stopped
    # 11 (if present) is the Pushsafer answer, 1 = Answer, 0 = no possibilty to answer

    try:
        appkey  = addrs[0]
    except:
        srv.logging.warn("No pushsafer private or alias key configured for target `%s'" % (item.target))
        return False

    params = {
            'expire' : 3600,
        }

    if len(addrs) > 1:
        params['d'] = addrs[1]

    if len(addrs) > 2:
        params['i'] = addrs[2]

    if len(addrs) > 3:
        params['s'] = addrs[3]

    if len(addrs) > 3:
        params['v'] = addrs[4]

    if len(addrs) > 4:
        params['u'] = addrs[5]

    if len(addrs) > 5:
        params['ut'] = addrs[6]

    if len(addrs) > 6:
        params['l'] = addrs[7]

    if len(addrs) > 7:
        params['pr'] = addrs[8]

    if len(addrs) > 8:
        params['re'] = addrs[9]

    if len(addrs) > 9:
        params['ex'] = addrs[10]

    if len(addrs) > 10:
        params['a'] = addrs[11]

    if title is not None:
        params['t'] = title

    try:
        srv.logging.debug("Sending pushsafer notification to %s [%s]..." % (item.target, params))
        pushsafer(m=message, k=appkey, **params)
        srv.logging.debug("Successfully sent pushsafer notification")
    except Exception as e:
        srv.logging.warn("Error sending pushsafer notification to %s [%s]: %s" % (item.target, params, e))
        return False

    return True



import base64
import logging
import os
from unittest import mock

import responses
from requests_toolbelt import MultipartDecoder

from mqttwarn.util import load_module_from_file
from tests.util import ProcessorItem as Item


def add_successful_mock_response():
    responses.add(
        responses.POST,
        "https://api.pushover.net/1/messages.json",
        json={"status": 1},
        status=200,
    )


def add_failed_mock_response():
    responses.add(
        responses.POST,
        "https://api.pushover.net/1/messages.json",
        json={"status": 999},
        status=400,
    )


@responses.activate
def test_pushover_success(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2"],
        message="⚽ Notification message ⚽",
        data={},
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()
        outcome = module.plugin(srv, item)

        assert len(responses.calls) == 1
        assert (
            responses.calls[0].request.url == "https://api.pushover.net/1/messages.json"
        )
        assert (
            responses.calls[0].request.body
            == "user=userkey2&token=appkey2&retry=60&expire=3600&message=%E2%9A%BD+Notification+message+%E2%9A%BD"
        )
        assert responses.calls[0].request.headers["User-Agent"] == "mqttwarn"

        assert responses.calls[0].response.status_code == 200
        assert responses.calls[0].response.text == '{"status": 1}'

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


@responses.activate
@mock.patch.dict(os.environ, {"PUSHOVER_USER": "userkey2", "PUSHOVER_TOKEN": "appkey2"})
def test_pushover_success_with_credentials_from_environment(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=[None, None],
        message="⚽ Notification message ⚽",
        data={},
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()
        outcome = module.plugin(srv, item)

        assert len(responses.calls) == 1
        assert (
            responses.calls[0].request.url == "https://api.pushover.net/1/messages.json"
        )
        assert (
            responses.calls[0].request.body
            == "user=userkey2&token=appkey2&retry=60&expire=3600&message=%E2%9A%BD+Notification+message+%E2%9A%BD"
        )
        assert responses.calls[0].request.headers["User-Agent"] == "mqttwarn"

        assert responses.calls[0].response.status_code == 200
        assert responses.calls[0].response.text == '{"status": 1}'

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


@responses.activate
def test_pushover_success_with_sound(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2", "sound1"],
        message="⚽ Notification message ⚽",
        data={},
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()
        outcome = module.plugin(srv, item)

        assert (
            responses.calls[0].request.body
            == "user=userkey2&token=appkey2&retry=60&expire=3600&sound=sound1&message=%E2%9A%BD+Notification+message+%E2%9A%BD"
        )

        assert responses.calls[0].response.status_code == 200
        assert responses.calls[0].response.text == '{"status": 1}'

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


@responses.activate
def test_pushover_success_with_devices(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2", None, "cellphone1,cellphone2"],
        message="⚽ Notification message ⚽",
        data={},
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()
        outcome = module.plugin(srv, item)

        assert (
            responses.calls[0].request.body
            == "user=userkey2&token=appkey2&retry=60&expire=3600&devices=cellphone1%2Ccellphone2&message=%E2%9A%BD+Notification+message+%E2%9A%BD"
        )

        assert responses.calls[0].response.status_code == 200
        assert responses.calls[0].response.text == '{"status": 1}'

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


@responses.activate
def test_pushover_success_with_callback_and_title_and_priority_and_alternative_message(
    srv, caplog
):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={"callback": "https://example.org/pushover-callback"},
        target="test",
        addrs=["userkey2", "appkey2"],
        priority=555,
        title="⚽ Message title ⚽",
        message="⚽ Notification message ⚽",
        data={"message": "⚽ Alternative notification message ⚽"},
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()
        outcome = module.plugin(srv, item)

        assert (
            responses.calls[0].request.body
            == "user=userkey2&token=appkey2&retry=60&expire=3600&title=%E2%9A%BD+Message+title+%E2%9A%BD&priority=555&callback=https%3A%2F%2Fexample.org%2Fpushover-callback&message=%E2%9A%BD+Alternative+notification+message+%E2%9A%BD"
        )

        assert responses.calls[0].response.status_code == 200
        assert responses.calls[0].response.text == '{"status": 1}'

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


@responses.activate
def test_pushover_success_with_imageurl(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2"],
        message="⚽ Notification message ⚽",
        data={"imageurl": "https://example.org/image"},
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()

        image = open("./assets/pushover.png", "rb").read()
        responses.add(
            responses.GET,
            "https://example.org/image",
            body=image,
            stream=True,
            status=200,
        )

        outcome = module.plugin(srv, item)

        # Check response status.
        assert responses.calls[1].response.status_code == 200
        assert responses.calls[1].response.text == '{"status": 1}'

        # Decode multipart request.
        request = responses.calls[1].request
        decoder = MultipartDecoder(request.body, request.headers["Content-Type"])

        content_disposition_headers = []
        contents = {}
        for part in decoder.parts:
            content_disposition_headers.append(part.headers[b"Content-Disposition"])

            key = part.headers[b"Content-Disposition"]
            contents[key] = part.content

        # Proof request has all body parts.
        assert content_disposition_headers == [
            b'form-data; name="user"',
            b'form-data; name="token"',
            b'form-data; name="retry"',
            b'form-data; name="expire"',
            b'form-data; name="message"',
            b'form-data; name="attachment"; filename="image.jpg"',
        ]

        # Proof parameter body parts, modulo image content, have correct values.
        assert list(contents.values())[:-1] == [
            b"userkey2",
            b"appkey2",
            b"60",
            b"3600",
            b"\xe2\x9a\xbd Notification message \xe2\x9a\xbd",
        ]

        # Proof image has content.
        assert len(decoder.parts[-1].content) == 45628

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


@responses.activate
def test_pushover_success_with_imageurl_and_basic_authentication(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2"],
        message="⚽ Notification message ⚽",
        data={
            "imageurl": "https://example.org/image",
            "auth": "basic",
            "user": "foo",
            "password": "bar",
        },
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()

        image = open("./assets/pushover.png", "rb").read()
        responses.add(
            responses.GET,
            "https://example.org/image",
            body=image,
            stream=True,
            status=200,
        )

        outcome = module.plugin(srv, item)

        # Proof authentication on image request.
        assert (
            responses.calls[0].request.headers["Authorization"] == "Basic Zm9vOmJhcg=="
        )

        # Check response status.
        assert responses.calls[1].response.status_code == 200
        assert responses.calls[1].response.text == '{"status": 1}'

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


@responses.activate
def test_pushover_success_with_imageurl_and_digest_authentication(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2"],
        message="⚽ Notification message ⚽",
        data={
            "imageurl": "https://example.org/image",
            "auth": "digest",
            "user": "foo",
            "password": "bar",
        },
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()

        image = open("./assets/pushover.png", "rb").read()
        responses.add(
            responses.GET,
            "https://example.org/image",
            body=image,
            stream=True,
            status=200,
        )

        outcome = module.plugin(srv, item)

        # Proof authentication on image request.
        # FIXME: Currently not possible because Digest auth will only work if
        #        the server answers with 4xx.
        # assert (
        #    responses.calls[0].request.headers["Authorization"] == "Digest something"
        # )

        # Check response status.
        assert responses.calls[1].response.status_code == 200
        assert responses.calls[1].response.text == '{"status": 1}'

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


@responses.activate
def test_pushover_success_with_imagebase64(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    image = open("./assets/pushover.png", "rb").read()
    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2"],
        message="⚽ Notification message ⚽",
        data={"imagebase64": base64.encodebytes(image)},
    )

    with caplog.at_level(logging.DEBUG):

        add_successful_mock_response()
        outcome = module.plugin(srv, item)

        # Check response status.
        assert responses.calls[0].response.status_code == 200
        assert responses.calls[0].response.text == '{"status": 1}'

        # Decode multipart request.
        request = responses.calls[0].request
        decoder = MultipartDecoder(request.body, request.headers["Content-Type"])

        content_disposition_headers = []
        contents = {}
        for part in decoder.parts:
            content_disposition_headers.append(part.headers[b"Content-Disposition"])

            key = part.headers[b"Content-Disposition"]
            contents[key] = part.content

        # Proof request has all body parts.
        assert content_disposition_headers == [
            b'form-data; name="user"',
            b'form-data; name="token"',
            b'form-data; name="retry"',
            b'form-data; name="expire"',
            b'form-data; name="message"',
            b'form-data; name="attachment"; filename="image.jpg"',
        ]

        # Proof parameter body parts, modulo image content, have correct values.
        assert list(contents.values())[:-1] == [
            b"userkey2",
            b"appkey2",
            b"60",
            b"3600",
            b"\xe2\x9a\xbd Notification message \xe2\x9a\xbd",
        ]

        # Proof image has content.
        assert len(decoder.parts[-1].content) == 45628

        assert outcome is True
        assert "Sending pushover notification to test" in caplog.text
        assert "Successfully sent pushover notification" in caplog.text


def test_pushover_failure_invalid_configuration(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=[None],
    )

    with caplog.at_level(logging.DEBUG):
        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "Invalid address configuration for target `test'" in caplog.text


def test_pushover_failure_missing_credentials(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=[None, None],
        data={},
    )

    with caplog.at_level(logging.DEBUG):
        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "No pushover credentials configured for target `test'" in caplog.text


@responses.activate
def test_pushover_failure_request_error(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2"],
        message="⚽ Notification message ⚽",
        data={},
    )

    with caplog.at_level(logging.DEBUG):

        # Make remote call bail out.
        module.pushover = mock.MagicMock(side_effect=Exception("something failed"))

        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "Sending pushover notification to test" in caplog.text
        assert "Error sending pushover notification: something failed" in caplog.text


@responses.activate
def test_pushover_failure_response_error(srv, caplog):

    module = load_module_from_file("mqttwarn/services/pushover.py")

    item = Item(
        config={},
        target="test",
        addrs=["userkey2", "appkey2"],
        message="⚽ Notification message ⚽",
        data={},
    )

    with caplog.at_level(logging.DEBUG):

        add_failed_mock_response()
        outcome = module.plugin(srv, item)

        assert len(responses.calls) == 1

        assert (
            responses.calls[0].request.url == "https://api.pushover.net/1/messages.json"
        )
        assert (
            responses.calls[0].request.body
            == "user=userkey2&token=appkey2&retry=60&expire=3600&message=%E2%9A%BD+Notification+message+%E2%9A%BD"
        )
        assert responses.calls[0].request.headers["User-Agent"] == "mqttwarn"

        assert responses.calls[0].response.status_code == 400
        assert responses.calls[0].response.text == '{"status": 999}'

        assert outcome is False
        assert "Sending pushover notification to test" in caplog.text
        assert (
            "Error sending pushover notification: b'{\"status\": 999}'" in caplog.text
        )



# -*- coding: utf-8 -*-
# (c) 2021 The mqttwarn developers
import logging
from unittest import mock
from unittest.mock import call, PropertyMock

from surrogate import surrogate

from mqttwarn.util import load_module_by_name, load_module_from_file
from tests.util import ProcessorItem as Item


def test_alexa_notify_me_success(srv, caplog):

    module = load_module_from_file("mqttwarn/services/alexa-notify-me.py")

    accessCode = "myToken"
    item = Item(addrs=[accessCode], message="⚽ Notification message ⚽")

    with caplog.at_level(logging.DEBUG):
        with mock.patch("requests.post") as requests_mock:
            outcome = module.plugin(srv, item)
            requests_mock.assert_called_once_with(
                url="https://api.notifymyecho.com/v1/NotifyMe",
                data='{"notification": "\\u26bd Notification message \\u26bd", "accessCode": "myToken"}',
            )

        assert outcome is True
        assert "Sending to NotifyMe service" in caplog.text
        assert "Successfully sent to NotifyMe service" in caplog.text


def test_alexa_notify_me_real_auth_failure(srv, caplog):
    module = load_module_from_file("mqttwarn/services/alexa-notify-me.py")

    accessCode = "myToken"
    item = Item(addrs=[accessCode], message="⚽ Notification message ⚽")

    with caplog.at_level(logging.DEBUG):
        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "Sending to NotifyMe service" in caplog.text
        assert "Failed to send message to NotifyMe service" in caplog.text


@surrogate("puka")
@mock.patch("puka.Client", create=True)
def test_amqp_success(mock_puka_client, srv, caplog):
    module = load_module_by_name("mqttwarn.services.amqp")

    exchange, routing_key = ["name_of_exchange", "my_routing_key"]
    item = Item(
        config={"uri": "amqp://user:password@localhost:5672/"},
        target="test",
        addrs=[exchange, routing_key],
        message="⚽ Notification message ⚽",
    )

    with caplog.at_level(logging.DEBUG):

        outcome = module.plugin(srv, item)

        assert mock_puka_client.mock_calls == [
            mock.call("amqp://user:password@localhost:5672/"),
            call().connect(),
            call().wait(mock.ANY),
            call().basic_publish(
                exchange="name_of_exchange",
                routing_key="my_routing_key",
                headers={
                    "content_type": "text/plain",
                    "x-agent": "mqttwarn",
                    "delivery_mode": 1,
                },
                body="⚽ Notification message ⚽",
            ),
            call().wait(mock.ANY),
            call().close(),
        ]

        assert outcome is True
        assert "AMQP publish to test [name_of_exchange/my_routing_key]" in caplog.text
        assert "Successfully published AMQP notification" in caplog.text


@surrogate("puka")
def test_amqp_failure(srv, caplog):
    module = load_module_by_name("mqttwarn.services.amqp")

    exchange, routing_key = ["name_of_exchange", "my_routing_key"]
    item = Item(
        config={"uri": "amqp://user:password@localhost:5672/"},
        target="test",
        addrs=[exchange, routing_key],
        message="⚽ Notification message ⚽",
    )

    with caplog.at_level(logging.DEBUG):

        mock_connection = mock.MagicMock()

        # Make the call to `basic_publish` raise an exception.
        def error(*args, **kwargs):
            raise Exception("something failed")

        mock_connection.basic_publish = error

        with mock.patch(
            "puka.Client", side_effect=[mock_connection], create=True
        ) as mock_client:

            outcome = module.plugin(srv, item)

            assert mock_client.mock_calls == [
                mock.call("amqp://user:password@localhost:5672/"),
            ]
            assert mock_connection.mock_calls == [
                call.connect(),
                call.wait(mock.ANY),
            ]

            assert outcome is False
            assert (
                "AMQP publish to test [name_of_exchange/my_routing_key]" in caplog.text
            )
            assert (
                "Error on AMQP publish to test [name_of_exchange/my_routing_key]: something failed"
                in caplog.text
            )


@surrogate("apns")
@mock.patch("apns.APNs", create=True)
@mock.patch("apns.Payload", create=True)
def test_apns_success(mock_apns_payload, mock_apns, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/apns.py")

        cert_file, key_file = ["cert_file", "key_file"]
        item = Item(
            target="test",
            addrs=[cert_file, key_file],
            message="⚽ Notification message ⚽",
            data={"apns_token": "foobar", "payload": "{}"},
        )

        outcome = module.plugin(srv, item)

        assert mock_apns_payload.mock_calls == [
            call(
                alert="⚽ Notification message ⚽",
                custom={},
                sound="default",
                badge=1,
            ),
        ]
        assert mock_apns.mock_calls == [
            mock.call(use_sandbox=False, cert_file="cert_file", key_file="key_file"),
            call().gateway_server.send_notification("foobar", mock.ANY),
        ]

        assert outcome is True
        assert "Successfully published APNS notification to foobar" in caplog.text


@surrogate("apns")
@mock.patch("apns.APNs", create=True)
@mock.patch("apns.Payload", create=True)
def test_apns_success_no_payload(mock_apns_payload, mock_apns, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/apns.py")

        cert_file, key_file = ["cert_file", "key_file"]
        item = Item(
            target="test",
            addrs=[cert_file, key_file],
            message="⚽ Notification message ⚽",
            data={"apns_token": "foobar"},
        )

        outcome = module.plugin(srv, item)

        assert mock_apns_payload.mock_calls == [
            call(
                alert="⚽ Notification message ⚽",
                custom={},
                sound="default",
                badge=1,
            ),
        ]
        assert mock_apns.mock_calls == [
            mock.call(use_sandbox=False, cert_file="cert_file", key_file="key_file"),
            call().gateway_server.send_notification("foobar", mock.ANY),
        ]

        assert outcome is True
        assert "Successfully published APNS notification to foobar" in caplog.text


@surrogate("apns")
@mock.patch("apns.APNs", create=True)
@mock.patch("apns.Payload", create=True)
def test_apns_success_custom_payload(mock_apns_payload, mock_apns, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/apns.py")

        cert_file, key_file = ["cert_file", "key_file"]
        item = Item(
            target="test",
            addrs=[cert_file, key_file],
            message="⚽ Notification message ⚽",
            data={
                "apns_token": "foobar",
                "payload": '{"custom": {"baz": "qux"}}',
            },
        )

        outcome = module.plugin(srv, item)

        assert mock_apns_payload.mock_calls == [
            call(
                alert="⚽ Notification message ⚽",
                custom={"baz": "qux"},
                sound="default",
                badge=1,
            ),
        ]
        assert mock_apns.mock_calls == [
            mock.call(use_sandbox=False, cert_file="cert_file", key_file="key_file"),
            call().gateway_server.send_notification("foobar", mock.ANY),
        ]

        assert outcome is True
        assert "Successfully published APNS notification to foobar" in caplog.text


@surrogate("apns")
@mock.patch("apns.APNs", create=True)
@mock.patch("apns.Payload", create=True)
def test_apns_failure_invalid_config(mock_apns_payload, mock_apns, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/apns.py")

        item = Item(
            target="test",
            addrs=[None],
            message="⚽ Notification message ⚽",
            data={"apns_token": "foobar", "payload": "{}"},
        )

        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "Incorrect service configuration" in caplog.text


@surrogate("apns")
@mock.patch("apns.APNs", create=True)
@mock.patch("apns.Payload", create=True)
def test_apns_failure_apns_token_missing(mock_apns_payload, mock_apns, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/apns.py")

        cert_file, key_file = ["cert_file", "key_file"]
        item = Item(
            target="test",
            addrs=[cert_file, key_file],
            message="⚽ Notification message ⚽",
            data={},
        )

        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "Cannot notify via APNS: apns_token is missing" in caplog.text


@surrogate("apprise")
@mock.patch("apprise.Apprise", create=True)
@mock.patch("apprise.AppriseAsset", create=True)
def test_apprise_success(apprise_asset, apprise_mock, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/apprise.py")

        item = Item(
            config={
                "baseuri": "mailtos://smtp_username:smtp_password@mail.example.org"
            },
            target="test",
            addrs=["foo@example.org", "bar@example.org"],
            title="⚽ Message title ⚽",
            message="⚽ Notification message ⚽",
        )

        outcome = module.plugin(srv, item)

        assert apprise_mock.mock_calls == [
            call(asset=mock.ANY),
            call().add(
                "mailtos://smtp_username:smtp_password@mail.example.org?from=None&to=foo@example.org,bar@example.org"
            ),
            call().notify(body="⚽ Notification message ⚽", title="⚽ Message title ⚽"),
            call().notify().__bool__(),
        ]

        assert outcome is True
        assert (
            "Sending notification to Apprise test, addresses: ['foo@example.org', 'bar@example.org']"
            in caplog.text
        )
        assert "Successfully sent message using Apprise" in caplog.text


@surrogate("apprise")
@mock.patch("apprise.Apprise", create=True)
@mock.patch("apprise.AppriseAsset", create=True)
def test_apprise_failure_no_addresses(apprise_asset, apprise_mock, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/apprise.py")

        item = Item(
            config={
                "baseuri": "mailtos://smtp_username:smtp_password@mail.example.org"
            },
            target="test",
            addrs=[],
            title="⚽ Message title ⚽",
            message="⚽ Notification message ⚽",
        )

        outcome = module.plugin(srv, item)

        assert apprise_mock.mock_calls == []

        assert outcome is False
        assert (
            "Skipped sending notification to Apprise test, no addresses configured"
            in caplog.text
        )


@surrogate("apprise")
def test_apprise_failure_notify(srv, caplog):

    with caplog.at_level(logging.DEBUG):

        mock_connection = mock.MagicMock()

        # Make the call to `notify` signal failure.
        def error(*args, **kwargs):
            return False

        mock_connection.notify = error

        with mock.patch(
            "apprise.Apprise", side_effect=[mock_connection], create=True
        ) as mock_client:
            with mock.patch("apprise.AppriseAsset", create=True) as mock_asset:
                module = load_module_from_file("mqttwarn/services/apprise.py")

                item = Item(
                    config={
                        "baseuri": "mailtos://smtp_username:smtp_password@mail.example.org"
                    },
                    target="test",
                    addrs=["foo@example.org", "bar@example.org"],
                    title="⚽ Message title ⚽",
                    message="⚽ Notification message ⚽",
                )

                outcome = module.plugin(srv, item)

                assert mock_client.mock_calls == [
                    mock.call(asset=mock.ANY),
                ]
                assert mock_connection.mock_calls == [
                    call.add(
                        "mailtos://smtp_username:smtp_password@mail.example.org?from=None&to=foo@example.org,bar@example.org"
                    ),
                ]

                assert outcome is False
                assert (
                    "Sending notification to Apprise test, addresses: ['foo@example.org', 'bar@example.org']"
                    in caplog.text
                )
                assert "Sending message using Apprise failed" in caplog.text


@surrogate("apprise")
def test_apprise_error(srv, caplog):

    with caplog.at_level(logging.DEBUG):

        mock_connection = mock.MagicMock()

        # Make the call to `notify` raise an exception.
        def error(*args, **kwargs):
            raise Exception("something failed")

        mock_connection.notify = error

        with mock.patch(
            "apprise.Apprise", side_effect=[mock_connection], create=True
        ) as mock_client:
            with mock.patch("apprise.AppriseAsset", create=True) as mock_asset:
                module = load_module_from_file("mqttwarn/services/apprise.py")

                item = Item(
                    config={
                        "baseuri": "mailtos://smtp_username:smtp_password@mail.example.org"
                    },
                    target="test",
                    addrs=["foo@example.org", "bar@example.org"],
                    title="⚽ Message title ⚽",
                    message="⚽ Notification message ⚽",
                )

                outcome = module.plugin(srv, item)

                assert mock_client.mock_calls == [
                    mock.call(asset=mock.ANY),
                ]
                assert mock_connection.mock_calls == [
                    call.add(
                        "mailtos://smtp_username:smtp_password@mail.example.org?from=None&to=foo@example.org,bar@example.org"
                    ),
                ]

                assert outcome is False
                assert (
                    "Sending notification to Apprise test, addresses: ['foo@example.org', 'bar@example.org']"
                    in caplog.text
                )
                assert "Error sending message to test: something failed" in caplog.text


@surrogate("apprise")
@mock.patch("apprise.Apprise", create=True)
@mock.patch("apprise.AppriseAsset", create=True)
def test_apprise_success_with_sender(apprise_asset, apprise_mock, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/apprise.py")

        item = Item(
            config={
                "baseuri": "mailtos://smtp_username:smtp_password@mail.example.org",
                "sender": "example@example.org",
                "sender_name": "Max Mustermann",
            },
            target="test",
            addrs=["foo@example.org", "bar@example.org"],
            title="⚽ Message title ⚽",
            message="⚽ Notification message ⚽",
        )

        outcome = module.plugin(srv, item)

        assert apprise_mock.mock_calls == [
            call(asset=mock.ANY),
            call().add(
                "mailtos://smtp_username:smtp_password@mail.example.org?from=example@example.org&to=foo@example.org,bar@example.org&name=Max Mustermann"
            ),
            call().notify(body="⚽ Notification message ⚽", title="⚽ Message title ⚽"),
            call().notify().__bool__(),
        ]

        assert outcome is True
        assert "Successfully sent message using Apprise" in caplog.text


@surrogate("asterisk.manager")
@mock.patch("asterisk.manager.Manager", create=True)
def test_asterisk_success(asterisk_mock, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        attrs = {"login.return_value": 42, "originate.return_value": 42}
        asterisk_mock.return_value = mock.MagicMock(**attrs)

        module = load_module_from_file("mqttwarn/services/asterisk.py")

        item = Item(
            config={
                "host": "asterisk.example.org",
                "port": 5038,
                "username": "foobar",
                "password": "bazqux",
                "extension": 2222,
                "context": "default",
            },
            target="test",
            addrs=["SIP/avaya/", "0123456789"],
            message="⚽ Notification message ⚽",
        )

        outcome = module.plugin(srv, item)

        assert asterisk_mock.mock_calls == [
            call(),
            call().connect("asterisk.example.org", 5038),
            call().login("foobar", "bazqux"),
            call().originate(
                "SIP/avaya/0123456789",
                2222,
                context="default",
                priority="1",
                caller_id=2222,
                variables={"text": "⚽ Notification message ⚽"},
            ),
            call().logoff(),
            call().close(),
        ]

        assert outcome is True
        assert "Authentication 42" in caplog.text
        assert "Call 42" in caplog.text


class ManagerSocketException(Exception):
    pass


class ManagerAuthException(Exception):
    pass


class ManagerException(Exception):
    pass


@surrogate("asterisk.manager")
@mock.patch("asterisk.manager.Manager", create=True)
@mock.patch(
    "asterisk.manager.ManagerSocketException", ManagerSocketException, create=True
)
def test_asterisk_failure_no_connection(asterisk_mock, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        attrs = {"connect.side_effect": ManagerSocketException("something failed")}
        asterisk_mock.return_value = mock.MagicMock(**attrs)

        module = load_module_from_file("mqttwarn/services/asterisk.py")

        item = Item(
            config={
                "host": "asterisk.example.org",
                "port": 5038,
                "username": "foobar",
                "password": "bazqux",
                "extension": 2222,
                "context": "default",
            },
            target="test",
            addrs=["SIP/avaya/", "0123456789"],
            message="⚽ Notification message ⚽",
        )

        outcome = module.plugin(srv, item)

        assert asterisk_mock.mock_calls == [
            call(),
            call().connect("asterisk.example.org", 5038),
            call().close(),
        ]

        assert outcome is False
        assert "Error connecting to the manager: something failed" in caplog.text


@surrogate("asterisk.manager")
@mock.patch("asterisk.manager.Manager", create=True)
@mock.patch(
    "asterisk.manager.ManagerSocketException", ManagerSocketException, create=True
)
@mock.patch("asterisk.manager.ManagerAuthException", ManagerAuthException, create=True)
def test_asterisk_failure_login_invalid(asterisk_mock, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        attrs = {"login.side_effect": ManagerAuthException("something failed")}
        asterisk_mock.return_value = mock.MagicMock(**attrs)

        module = load_module_from_file("mqttwarn/services/asterisk.py")

        item = Item(
            config={
                "host": "asterisk.example.org",
                "port": 5038,
                "username": "foobar",
                "password": "bazqux",
                "extension": 2222,
                "context": "default",
            },
            target="test",
            addrs=["SIP/avaya/", "0123456789"],
            message="⚽ Notification message ⚽",
        )

        outcome = module.plugin(srv, item)

        assert asterisk_mock.mock_calls == [
            call(),
            call().connect("asterisk.example.org", 5038),
            call().login("foobar", "bazqux"),
            call().close(),
        ]

        assert outcome is False
        assert "Error logging in to the manager: something failed" in caplog.text


@surrogate("asterisk.manager")
@mock.patch("asterisk.manager.Manager", create=True)
@mock.patch(
    "asterisk.manager.ManagerSocketException", ManagerSocketException, create=True
)
@mock.patch("asterisk.manager.ManagerAuthException", ManagerAuthException, create=True)
@mock.patch("asterisk.manager.ManagerException", ManagerException, create=True)
def test_asterisk_failure_originate_croaks(asterisk_mock, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        attrs = {
            "login.return_value": 42,
            "originate.side_effect": ManagerException("something failed"),
        }
        asterisk_mock.return_value = mock.MagicMock(**attrs)

        module = load_module_from_file("mqttwarn/services/asterisk.py")

        item = Item(
            config={
                "host": "asterisk.example.org",
                "port": 5038,
                "username": "foobar",
                "password": "bazqux",
                "extension": 2222,
                "context": "default",
            },
            target="test",
            addrs=["SIP/avaya/", "0123456789"],
            message="⚽ Notification message ⚽",
        )

        outcome = module.plugin(srv, item)

        assert asterisk_mock.mock_calls == [
            call(),
            call().connect("asterisk.example.org", 5038),
            call().login("foobar", "bazqux"),
            call().originate(
                "SIP/avaya/0123456789",
                2222,
                context="default",
                priority="1",
                caller_id=2222,
                variables={"text": "⚽ Notification message ⚽"},
            ),
            call().close(),
        ]

        assert outcome is False
        assert "Error: something failed" in caplog.text


@surrogate("asterisk.manager")
@mock.patch("asterisk.manager.Manager", create=True)
@mock.patch(
    "asterisk.manager.ManagerSocketException", ManagerSocketException, create=True
)
@mock.patch("asterisk.manager.ManagerAuthException", ManagerAuthException, create=True)
@mock.patch("asterisk.manager.ManagerException", ManagerException, create=True)
def test_asterisk_success_with_broken_close(asterisk_mock, srv, caplog):

    with caplog.at_level(logging.DEBUG):

        attrs = {
            "login.return_value": 42,
            "originate.return_value": 42,
            "close.side_effect": ManagerSocketException("something failed"),
        }
        asterisk_mock.return_value = mock.MagicMock(**attrs)

        module = load_module_from_file("mqttwarn/services/asterisk.py")

        item = Item(
            config={
                "host": "asterisk.example.org",
                "port": 5038,
                "username": "foobar",
                "password": "bazqux",
                "extension": 2222,
                "context": "default",
            },
            target="test",
            addrs=["SIP/avaya/", "0123456789"],
            message="⚽ Notification message ⚽",
        )

        outcome = module.plugin(srv, item)

        assert asterisk_mock.mock_calls == [
            call(),
            call().connect("asterisk.example.org", 5038),
            call().login("foobar", "bazqux"),
            call().originate(
                "SIP/avaya/0123456789",
                2222,
                context="default",
                priority="1",
                caller_id=2222,
                variables={"text": "⚽ Notification message ⚽"},
            ),
            call().logoff(),
            call().close(),
        ]

        assert outcome is True


def test_autoremote_success(srv, caplog):

    item = Item(
        target="test",
        addrs=["ApiKey", "Password", "Target", "Group", "TTL"],
        topic="autoremote/user",
        message="⚽ Notification message ⚽",
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/autoremote.py")

        with mock.patch("requests.get") as requests_mock:
            outcome = module.plugin(srv, item)
            requests_mock.assert_called_once_with(
                "https://autoremotejoaomgcd.appspot.com/sendmessage",
                params={
                    "key": "ApiKey",
                    "message": "⚽ Notification message ⚽",
                    "target": "Target",
                    "sender": "autoremote/user",
                    "password": "Password",
                    "ttl": "TTL",
                    "collapseKey": "Group",
                },
            )

        assert outcome is True
        assert "Sending to autoremote service" in caplog.text
        assert "Successfully sent to autoremote service" in caplog.text


def test_autoremote_failure(srv, caplog):

    item = Item(
        target="test",
        addrs=["ApiKey", "Password", "Target", "Group", "TTL"],
        topic="autoremote/user",
        message="⚽ Notification message ⚽",
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/autoremote.py")

        with mock.patch(
            "requests.get", side_effect=Exception("something failed")
        ) as requests_mock:
            outcome = module.plugin(srv, item)
            requests_mock.assert_called_once_with(
                "https://autoremotejoaomgcd.appspot.com/sendmessage",
                params={
                    "key": "ApiKey",
                    "message": "⚽ Notification message ⚽",
                    "target": "Target",
                    "sender": "autoremote/user",
                    "password": "Password",
                    "ttl": "TTL",
                    "collapseKey": "Group",
                },
            )

        assert outcome is False
        assert "Sending to autoremote service" in caplog.text
        assert (
            "Failed to send message to autoremote service: something failed"
            in caplog.text
        )


def test_azure_iot_success_string(srv, caplog):

    item = Item(
        config={"iothubname": "acmehub"},
        target="test",
        addrs=["device-id", "SharedAccessSignature sr=..."],
        message="⚽ Notification message ⚽",
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/azure_iot.py")

        mqtt_publish_mock = mock.MagicMock()
        module.mqtt = mqtt_publish_mock

        outcome = module.plugin(srv, item)
        mqtt_publish_mock.single.assert_called_once_with(
            "devices/device-id/messages/events/",
            bytearray(b"\xe2\x9a\xbd Notification message \xe2\x9a\xbd"),
            auth={
                "username": "acmehub.azure-devices.net/device-id/?api-version=2018-06-30",
                "password": "SharedAccessSignature sr=...",
            },
            tls={
                "ca_certs": None,
                "certfile": None,
                "keyfile": None,
                "tls_version": mock.ANY,
                "ciphers": None,
                "cert_reqs": mock.ANY,
            },
            hostname="acmehub.azure-devices.net",
            port=8883,
            protocol=4,
            qos=0,
            retain=False,
            client_id="device-id",
        )

        assert outcome is True
        assert (
            "Publishing to Azure IoT Hub for target=test (device-id): devices/device-id/messages/events/ 'bytearray(b'\\xe2\\x9a\\xbd Notification message \\xe2\\x9a\\xbd')'"
            in caplog.text
        )


def test_azure_iot_success_bytes(srv, caplog):

    item = Item(
        config={"iothubname": "acmehub"},
        target="test",
        addrs=["device-id", "SharedAccessSignature sr=..."],
        message=b"### Notification message ###",
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/azure_iot.py")

        mqtt_publish_mock = mock.MagicMock()
        module.mqtt = mqtt_publish_mock

        outcome = module.plugin(srv, item)
        mqtt_publish_mock.single.assert_called_once_with(
            "devices/device-id/messages/events/",
            bytearray(b"### Notification message ###"),
            auth={
                "username": "acmehub.azure-devices.net/device-id/?api-version=2018-06-30",
                "password": "SharedAccessSignature sr=...",
            },
            tls={
                "ca_certs": None,
                "certfile": None,
                "keyfile": None,
                "tls_version": mock.ANY,
                "ciphers": None,
                "cert_reqs": mock.ANY,
            },
            hostname="acmehub.azure-devices.net",
            port=8883,
            protocol=4,
            qos=0,
            retain=False,
            client_id="device-id",
        )

        assert outcome is True
        assert (
            "Publishing to Azure IoT Hub for target=test (device-id): devices/device-id/messages/events/ 'b'### Notification message ###''"
            in caplog.text
        )


def test_azure_iot_failure_wrong_qos(srv, caplog):

    item = Item(
        config={"iothubname": "acmehub", "qos": 999},
        target="test",
        addrs=["device-id", "SharedAccessSignature sr=..."],
        message="⚽ Notification message ⚽",
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/azure_iot.py")

        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "Only QoS 0 or 1 allowed for Azure IoT Hub, not '999'" in caplog.text


def test_azure_iot_failure_invalid_message(srv, caplog):

    item = Item(
        config={"iothubname": "acmehub"},
        target="test",
        addrs=["device-id", "SharedAccessSignature sr=..."],
    )

    with mock.patch.object(Item, "message", new_callable=PropertyMock) as msg_mock:
        msg_mock.side_effect = Exception("something failed")

        with caplog.at_level(logging.DEBUG):

            module = load_module_from_file("mqttwarn/services/azure_iot.py")

            outcome = module.plugin(srv, item)

            assert outcome is False
            assert (
                "Unable to prepare message for target=test: something failed"
                in caplog.text
            )


def test_azure_iot_failure_mqtt_publish(srv, caplog):

    item = Item(
        config={"iothubname": "acmehub"},
        target="test",
        addrs=["device-id", "SharedAccessSignature sr=..."],
        message="⚽ Notification message ⚽",
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/azure_iot.py")

        mqtt_publish_mock = mock.MagicMock(side_effect=Exception("something failed"))
        module.mqtt.single = mqtt_publish_mock

        outcome = module.plugin(srv, item)

        assert outcome is False
        assert (
            "Unable to publish to Azure IoT Hub for target=test (device-id): something failed"
            in caplog.text
        )


def test_carbon_success_metric_value_timestamp(srv, caplog):

    item = Item(
        target="test", addrs=["localhost", 2003], message="foo 42.42 1623887596", data={}
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        socket_mock = mock.MagicMock()
        module.socket.socket = socket_mock

        outcome = module.plugin(srv, item)
        assert socket_mock.mock_calls == [
            call(),
            call().connect(("localhost", 2003)),
            call().sendall("foo 42.42 1623887596\n"),
            call().close(),
        ]

        assert outcome is True
        assert "Sending to carbon: foo 42.42 1623887596" in caplog.text


def test_carbon_success_metric_value(srv, caplog):

    item = Item(
        target="test", addrs=["localhost", 2003], message="foo 42.42", data={}
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        socket_mock = mock.MagicMock()
        module.socket.socket = socket_mock

        outcome = module.plugin(srv, item)
        assert socket_mock.mock_calls == [
            call(),
            call().connect(("localhost", 2003)),
            call().sendall(mock.ANY),
            call().close(),
        ]

        assert outcome is True
        assert "Sending to carbon: foo 42.42" in caplog.text


def test_carbon_success_value(srv, caplog):

    item = Item(
        target="test", addrs=["localhost", 2003], message="42.42", data={}
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        socket_mock = mock.MagicMock()
        module.socket.socket = socket_mock

        outcome = module.plugin(srv, item)
        assert socket_mock.mock_calls == [
            call(),
            call().connect(("localhost", 2003)),
            call().sendall(mock.ANY),
            call().close(),
        ]

        assert outcome is True
        assert "Sending to carbon: ohno 42.42" in caplog.text


def test_carbon_success_value_metric_from_topic(srv, caplog):

    item = Item(
        target="test", addrs=["localhost", 2003], message="42.42", data={"topic": "foo/bar"}
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        socket_mock = mock.MagicMock()
        module.socket.socket = socket_mock

        outcome = module.plugin(srv, item)
        assert socket_mock.mock_calls == [
            call(),
            call().connect(("localhost", 2003)),
            call().sendall(mock.ANY),
            call().close(),
        ]

        assert outcome is True
        assert "Sending to carbon: foo.bar 42.42" in caplog.text


def test_carbon_success_value_metric_from_topic_with_leading_slash(srv, caplog):

    item = Item(
        target="test", addrs=["localhost", 2003], message="42.42", data={"topic": "/foo/bar"}
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        socket_mock = mock.MagicMock()
        module.socket.socket = socket_mock

        outcome = module.plugin(srv, item)
        assert socket_mock.mock_calls == [
            call(),
            call().connect(("localhost", 2003)),
            call().sendall(mock.ANY),
            call().close(),
        ]

        assert outcome is True
        assert "Sending to carbon: foo.bar 42.42" in caplog.text


def test_carbon_failure_invalid_configuration(srv, caplog):

    item = Item(target="test", addrs=["172.16.153.110", "foobar"])

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "Configuration for target `carbon' is incorrect" in caplog.text


def test_carbon_failure_empty_message(srv, caplog):

    item = Item(target="test", addrs=["172.16.153.110", 2003])

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "target `carbon': cannot split string" in caplog.text


def test_carbon_failure_invalid_message_format(srv, caplog):

    item = Item(
        target="test",
        addrs=["172.16.153.110", 2003],
        message="foo bar baz qux",
        data={},
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        outcome = module.plugin(srv, item)

        assert outcome is False
        assert "target `carbon': error decoding message" in caplog.text


def test_carbon_failure_connect(srv, caplog):

    item = Item(
        target="test", addrs=["localhost", 2003], message="foo 42.42 1623887596", data={}
    )

    with caplog.at_level(logging.DEBUG):

        module = load_module_from_file("mqttwarn/services/carbon.py")

        attrs = {"connect.side_effect": Exception("something failed")}
        socket_mock = mock.MagicMock()
        socket_mock.return_value = mock.MagicMock(**attrs)
        module.socket.socket = socket_mock

        outcome = module.plugin(srv, item)
        assert socket_mock.mock_calls == [call(), call().connect(("localhost", 2003))]

        assert outcome is False
        assert "Sending to carbon: foo 42.42 1623887596" in caplog.text
        assert (
            "Cannot send to carbon service localhost:2003: something failed"
            in caplog.text
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from pync import Notifier


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    text = item.message
    application_name = item.get('title', item.topic)

    # If item.data contains a URL field, use it as a target for the notification
    url = None
    extra_data = item.data
    if extra_data is not None:
        url = extra_data.get('url', None)

    try:
        Notifier.notify(text,  title=application_name, open=url)
    except Exception as e:
        srv.logging.warning("Cannot invoke Notifier to osx: %s" % e)
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2018-2020 The mqttwarn developers
import io
import os
import json
import tempfile

from builtins import str
import logging

import pytest

from mqttwarn.core import make_service, decode_payload
from tests import configfile_full, configfile_service_loading, configfile_no_functions, configfile_unknown_functions, \
    configfile_empty_functions
from tests.util import core_bootstrap, send_message


def test_make_service():
    service = make_service(name='foo')
    assert '<mqttwarn.core.Service object at' in str(service)


def test_bootstrap(caplog):
    """
    Bootstrap the core machinery without MQTT
    """

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile_full)

        # Proof that mqttwarn loaded all services properly
        assert 'Successfully loaded service "file"' in caplog.text, caplog.text
        assert 'Successfully loaded service "log"' in caplog.text, caplog.text

        assert 'Starting 1 worker threads' in caplog.text, caplog.text

        # Capturing the last message does not work. Why?
        #assert 'Job queue has 0 items to process' in caplog.text, caplog.text


def test_decode_payload_foo(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT.
        core_bootstrap(configfile=configfile_full)

        # Proof that decoding an unconfigured thing yields nothing sensible.
        outcome = decode_payload(section='foo', topic='bar', payload='baz')
        assert outcome['topic'] == 'bar'
        assert outcome['payload'] == 'baz'
        assert 'Cannot decode JSON object, payload=baz' in caplog.text, caplog.text


def test_decode_payload_json(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT.
        core_bootstrap(configfile=configfile_full)

        # Proof that decoding a valid JSON payload decodes it appropriately.
        outcome = decode_payload(section='foo', topic='bar', payload='{"baz": "qux"}')
        assert outcome['topic'] == 'bar'
        assert outcome['payload'] == '{"baz": "qux"}'
        assert outcome['baz'] == 'qux'


def test_decode_payload_datamap(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT.
        core_bootstrap(configfile=configfile_full)

        # Proof that decoding a valid JSON payload decodes it appropriately.
        outcome = decode_payload(section='test/datamap', topic='bar', payload='{"baz": "qux"}')
        assert outcome['topic'] == 'bar'
        assert outcome['baz'] == 'qux'
        assert outcome['datamap-key'] == 'datamap-value'


def test_decode_payload_alldata(caplog):

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT.
        core_bootstrap(configfile=configfile_full)

        # Proof that decoding a valid JSON payload decodes it appropriately.
        outcome = decode_payload(section='test/alldata', topic='bar', payload='{"baz": "qux"}')
        assert outcome['topic'] == 'bar'
        assert outcome['baz'] == 'qux'
        assert outcome['alldata-key'] == 'alldata-value'


def test_message_log(caplog):
    """
    Submit a message to the "log" plugin and proof
    everything gets dispatched properly.
    """

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile_full)

        # Signal mocked MQTT message to the core machinery for processing
        send_message(topic='test/log-1', payload='{"name": "temperature", "value": 42.42}')

        # Proof that the message has been routed to the "log" plugin properly
        assert "temperature: 42.42" in caplog.text, caplog.text


def test_message_file():
    """
    Submit a message to the "file" plugin and proof
    everything gets dispatched properly.
    """

    data = {
        'name': 'temperature',
        'value': 42.42,
    }

    tmpdir = tempfile.gettempdir()
    outputfile = os.path.join(tmpdir, 'mqttwarn-test.01')
    if os.path.exists(outputfile):
        os.unlink(outputfile)

    # Bootstrap the core machinery without MQTT.
    core_bootstrap(configfile=configfile_full)

    # Signal mocked MQTT message to the core machinery for processing.
    send_message(topic='test/file-1', payload=json.dumps(data))

    # Proof that the message has been written to the designated file properly.
    with open(outputfile) as f:
        content = f.read()
        assert "temperature: 42.42" in content, content


def test_message_file_unicode():
    """
    Submit a message to the "file" plugin and proof
    everything gets dispatched properly.

    This time, we use special characters (umlauts)
    to proof charset encoding is also handled properly.
    """

    data = {
        'item': 'Räuber Hotzenplotz'
    }

    tmpdir = tempfile.gettempdir()
    outputfile = os.path.join(tmpdir, 'mqttwarn-test.02')
    if os.path.exists(outputfile):
        os.unlink(outputfile)

    # Bootstrap the core machinery without MQTT.
    core_bootstrap(configfile=configfile_full)

    # Signal mocked MQTT message to the core machinery for processing.
    send_message(topic='test/file-2', payload=json.dumps(data))

    # Proof that the message has been written to the designated file properly.
    with io.open(outputfile, mode='rt', encoding='utf-8') as f:
        content = f.read()
        assert u'Räuber Hotzenplotz' in content, content


@pytest.mark.parametrize("configfile", [configfile_full, configfile_service_loading])
def test_plugin_module(caplog, configfile):
    """
    Check if loading a service module with dotted name works.
    """

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile)

        # Signal mocked MQTT message to the core machinery for processing
        send_message(topic='test/plugin-module', payload='{"name": "temperature", "value": 42.42}')

        # Proof that the message has been routed to the "log" plugin properly
        assert 'Plugin invoked' in caplog.text, caplog.text


@pytest.mark.parametrize("configfile", [configfile_full, configfile_service_loading])
def test_plugin_file(caplog, configfile):
    """
    Check if loading a service module from a file works.
    """

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile)

        # Signal mocked MQTT message to the core machinery for processing
        send_message(topic='test/plugin-file', payload='{"name": "temperature", "value": 42.42}')

        # Proof that the message has been routed to the "log" plugin properly
        assert 'Plugin invoked' in caplog.text, caplog.text


def test_xform_func(caplog):
    """
    Submit a message to the "log" plugin and proof
    everything gets dispatched properly.

    This time, it validates the "xform" function in the context of invoking
    a user-defined function defined through the "format" setting.
    """
    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile_full)

        # Signal mocked MQTT message to the core machinery for processing
        send_message(topic='test/log-2', payload='{"name": "temperature", "value": 42.42}')

        # Proof that the message has been routed to the "log" plugin properly
        assert "'value': 42.42" in caplog.text, caplog.text
        assert "'datamap-key': 'datamap-value'" in caplog.text, caplog.text


def test_config_no_functions(caplog):
    """
    Test a configuration file which has no `functions` setting.
    """

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile_no_functions)

        # Signal mocked MQTT message to the core machinery for processing
        send_message(topic='test/log-1', payload='{"name": "temperature", "value": 42.42}')

        # Proof that the message has been routed to the "log" plugin properly
        assert "temperature: 42.42" in caplog.text, caplog.text


def test_config_empty_functions(caplog):
    """
    Test a configuration file which has an empty `functions` setting.
    """

    with caplog.at_level(logging.DEBUG):

        # Bootstrap the core machinery without MQTT
        core_bootstrap(configfile=configfile_empty_functions)

        # Signal mocked MQTT message to the core machinery for processing
        send_message(topic='test/log-1', payload='{"name": "temperature", "value": 42.42}')

        # Proof that the message has been routed to the "log" plugin properly
        assert "temperature: 42.42" in caplog.text, caplog.text


def test_config_bad_functions(caplog):
    """
    Test a configuration file which has no `functions` setting.
    """

    with caplog.at_level(logging.DEBUG):

        # Bootstrapping the machinery with invalid path to functions file should croak.
        with pytest.raises(IOError) as excinfo:
            core_bootstrap(configfile=configfile_unknown_functions)

        error_message = str(excinfo.value)
        assert "UNKNOWN FILE REFERENCE" in error_message
        assert "not found" in error_message



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from six import string_types
from future import standard_library
standard_library.install_aliases()

import configparser
import paho.mqtt.publish as mqtt


def conf(ini_file, params):
    try:
        c = configparser.ConfigParser()
        f = open(ini_file, 'r')
        c.readfp(f)
        f.close()
    except Exception as e:
        raise

    if c.has_section('defaults'):
        # differentiate bool, int, str
        if c.has_option('defaults', 'hostname'):
            params['hostname'] = c.get('defaults', 'hostname')
        if c.has_option('defaults', 'client_id'):
            params['client_id'] = c.get('defaults', 'client_id')
        if c.has_option('defaults', 'port'):
            params['port'] = c.getint('defaults', 'port')
        if c.has_option('defaults', 'qos'):
            params['qos'] = c.getint('defaults', 'qos')
        if c.has_option('defaults', 'retain'):
            params['retain'] = c.getboolean('defaults', 'retain')

    auth = None
    if c.has_section('auth'):
        auth = dict(c.items('auth'))

    tls = None
    if c.has_section('tls'):
        tls = dict(c.items('tls'))

    return dict(connparams=params, auth=auth, tls=tls)


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    config = item.config

    hostname = config.get('hostname', 'localhost')
    port = int(config.get('port', '1883'))
    qos = int(config.get('qos', 0))
    retain = int(config.get('retain', 0))
    username = config.get('username', None)
    password = config.get('password', None)

    params = {
        'hostname': hostname,
        'port': port,
        'qos': qos,
        'retain': retain,
        'client_id': None,
    }

    auth = None
    tls = None

    if username is not None:
        auth = {
            'username': username,
            'password': password
        }

    ini_file = None
    try:
        outgoing_topic, ini_file = item.addrs
    except:
        outgoing_topic = item.addrs[0]

    if ini_file is not None:
        try:
            data = conf(ini_file, params)
        except Exception as e:
            srv.logging.error("Target mqtt cannot load/parse INI file `%s': %s", ini_file, e)
            return False

        if 'connparams' in data and data['connparams'] is not None:
            params = dict(list(params.items()) + list(data['connparams'].items()))

    # Attempt to interpolate data into topic name. If it isn't possible
    # ignore, and return without publish

    if item.data is not None:
        try:
            outgoing_topic = item.addrs[0].format(**item.data).encode('utf-8')
        except:
            srv.logging.debug("Outgoing topic cannot be formatted; not published")
            return False

    outgoing_payload = item.message
    if isinstance(outgoing_payload, string_types):
        outgoing_payload = bytearray(outgoing_payload, encoding='utf-8')

    try:
        mqtt.single(outgoing_topic, outgoing_payload,
                    auth=auth,
                    tls=tls,
                    **params)
    except Exception as e:
        srv.logging.warning("Cannot PUBlish via `mqtt:%s': %s" % (item.target, e))
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2018-2021 The mqttwarn developers
import time
from dataclasses import dataclass
from typing import Dict, Union, List

from paho.mqtt.client import MQTTMessage

from mqttwarn.configuration import load_configuration
from mqttwarn.core import bootstrap, on_message, load_services, start_workers


def core_bootstrap(configfile=None):
    """
    Bootstrap the core machinery without MQTT
    """

    # Load configuration file
    config = load_configuration(configfile)

    # Bootstrap mqttwarn.core
    bootstrap(config=config, scriptname='testdrive')

    # Load services
    services = config.getlist('defaults', 'launch')
    load_services(services)

    # Launch worker threads to operate on queue
    start_workers()


def send_message(topic=None, payload=None):

    # Mock an instance of an Eclipse Paho MQTTMessage
    message = MQTTMessage(mid=42, topic=topic.encode('utf-8'))
    if payload is not None:
        message.payload = payload.encode('utf-8')

    # Signal the message to the machinery
    on_message(None, None, message)

    # Give the machinery some time to process the message
    time.sleep(0.05)


@dataclass
class ProcessorItem:
    """
    A surrogate processor item for feeding into service handlers.
    """

    service: str = None
    target: str = None
    config: Dict = None
    addrs: List[str] = None
    priority: int = None
    topic: str = None
    title: str = None
    message: Union[str, bytes] = None
    data: Dict = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__author__    = 'Bram Hendrickx'
__copyright__ = 'Copyright 2016 Bram Hendrickx'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

Original script by Bram Hendrickx. https://github.com/jpmens/mqttwarn/blob/main/mqttwarn/services/ifttt.py
Modified to work with notify-me app for Alexa. http://www.thomptronics.com/notify-me
"""

__author__ = 'Michael Brougham'
__copyright__ = 'Copyright 2018 Michael Brougham'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import json
import requests


def plugin(srv, item):
    """ expects (key) in addrs """

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    try:
        srv.logging.debug("Sending to NotifyMe service.")
        body = json.dumps({
            "notification": item.message,
            "accessCode": item.addrs[0]
        })

        response = requests.post(url="https://api.notifymyecho.com/v1/NotifyMe", data=body)
        response.raise_for_status()

        srv.logging.debug("Successfully sent to NotifyMe service.")

    except Exception as e:
        srv.logging.warning("Failed to send message to NotifyMe service: %s" % e)
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2014-2019 The mqttwarn developers
from builtins import str
import os
import re
import imp
import sys
import json
import string
import traceback
import pkg_resources
from six import StringIO, string_types

try:
    import hashlib
    md = hashlib.md5
except ImportError:
    import md5
    md = md5.new


class Struct:
    """
    Convert Python dict to object?
    http://stackoverflow.com/questions/1305532/
    """
    def __init__(self, **entries):
        self.__dict__.update(entries)

    def __repr__(self):
        return '<%s>' % str("\n ".join("%s: %s" % (k, repr(v)) for (k, v) in list(self.__dict__.items())))

    def get(self, key, default=None):
        if key in self.__dict__ and self.__dict__[key] is not None:
            return self.__dict__[key]
        else:
            return default

    def enum(self):
        item = {}
        for (k, v) in list(self.__dict__.items()):
            item[k] = v
        return item


class Formatter(string.Formatter):
    """
    A custom string formatter. See also:
    - https://docs.python.org/2/library/string.html#format-string-syntax
    - https://docs.python.org/2/library/string.html#custom-string-formatting
    """

    def convert_field(self, value, conversion):
        """
        The conversion field causes a type coercion before formatting.
        By default, two conversion flags are supported: '!s' which calls
        str() on the value, and '!r' which calls repr().

        This also adds the '!j' conversion flag, which serializes the
        value to JSON format.

        See also https://github.com/jpmens/mqttwarn/issues/146.
        """
        if conversion == 'j':
            value = json.dumps(value)
        return value


def asbool(obj):
    """
    Shamelessly stolen from beaker.converters
    # (c) 2005 Ian Bicking and contributors; written for Paste (http://pythonpaste.org)
    # Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
    """
    if isinstance(obj, string_types):
        obj = obj.strip().lower()
        if obj in ['true', 'yes', 'on', 'y', 't', '1']:
            return True
        elif obj in ['false', 'no', 'off', 'n', 'f', '0']:
            return False
        else:
            raise ValueError(
                "String is not true/false: %r" % obj)
    return bool(obj)


def parse_cron_options(argstring):
    """
    Parse periodic task options.
    Obtains configuration value, returns dictionary.

    Example::

        my_periodic_task = 60; now=true

    Respective "argstring"::

        60; now=true

    """
    parts = argstring.split(';')
    options = {'interval': float(parts[0].strip())}
    for part in parts[1:]:
        name, value = part.split('=')
        options[name.strip()] = value.strip()
    return options


# http://code.activestate.com/recipes/473878-timeout-function-using-threading/
def timeout(func, args=(), kwargs={}, timeout_secs=10, default=False):
    import threading

    class InterruptableThread(threading.Thread):
        def __init__(self):
            threading.Thread.__init__(self)
            self.result = None
            self.exception = None

        def run(self):
            try:
                self.result = func(*args, **kwargs)

            # FIXME: Shouldn't we report this better?
            except Exception as ex:
                self.result = default
                self.exception = ex

    it = InterruptableThread()
    it.start()
    it.join(timeout_secs)

    if it.exception is not None:
        raise it.exception

    if it.is_alive():
        return default
    else:
        return it.result


def sanitize_function_name(s):
    func = None

    if s is not None:
        try:
            valid = re.match(r'^[\w]+\(\)', s)
            if valid is not None:
                func = re.sub('[()]', '', s)
        except:
            pass
    return func


def load_module_from_file(path):
    """
    http://code.davidjanes.com/blog/2008/11/27/how-to-dynamically-load-python-code/

    :param path:
    :return:
    """
    try:
        fp = open(path, 'rb')
        digest = md(path.encode('utf-8')).hexdigest()
        return imp.load_source(digest, path, fp)
    finally:
        try:
            fp.close()
        except:
            pass


def load_module_by_name(name):
    """
    https://pymotw.com/2/imp/#loading-modules

    :param name:
    :return:
    """
    module = import_module(name)
    return module


def import_module(name, path=None):
    """
    Derived from `import_from_dotted_path`:
    https://chase-seibert.github.io/blog/2014/04/23/python-imp-examples.html

    import_from_dotted_path('foo.bar') -> from foo import bar; return bar
    """

    try:
        next_module, remaining_names = name.split('.', 1)
    except ValueError:
        next_module = name
        remaining_names = None

    fp, pathname, description = imp.find_module(next_module, path)
    module = imp.load_module(next_module, fp, pathname, description)

    if remaining_names is None:
        return module
    if hasattr(module, remaining_names):
        return getattr(module, remaining_names)

    return import_module(remaining_names, path=module.__path__)


def load_functions(filepath=None):

    if not filepath:
        return None

    if not os.path.isfile(filepath):
        raise IOError("'{}' not found".format(filepath))

    mod_name, file_ext = os.path.splitext(os.path.split(filepath)[-1])

    if file_ext.lower() == '.py':
        py_mod = imp.load_source(mod_name, filepath)

    elif file_ext.lower() == '.pyc':
        py_mod = imp.load_compiled(mod_name, filepath)

    else:
        raise ValueError("'{}' does not have the .py or .pyc extension".format(filepath))

    return py_mod


def load_function(name=None, py_mod=None):
    assert name, 'Function name must be given'
    assert py_mod, 'Python module must be given'

    func = getattr(py_mod, name, None)

    if func is None:
        raise AttributeError("Function '{}' does not exist in '{}'".format(name, py_mod.__file__))

    return func


def get_resource_content(package, filename):
    with pkg_resources.resource_stream(package, filename) as stream:
        return stream.read().decode('utf-8')


def exception_traceback(exc_info=None):
    """
    Return a string containing a traceback message for the given
    exc_info tuple (as returned by sys.exc_info()).

    from setuptools.tests.doctest
    """

    if not exc_info:
        exc_info = sys.exc_info()

    # Get a traceback message.
    excout = StringIO()
    exc_type, exc_val, exc_tb = exc_info
    traceback.print_exception(exc_type, exc_val, exc_tb, file=excout)
    return excout.getvalue()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Text To Speech (TTS) for Chromecast devices, including Google Home Speakers

pip install pychromecast

See also https://github.com/skorokithakis/catt.
"""

__author__ = 'Chris Clark <clach04()gmail.com>'
__copyright__ = 'Copyright 2020 Chris Clark'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'


import os
from urllib.parse import urlencode


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    message = item.message
    mime_type = item.config.get('mimetype', 'audio/mp3')  # Google translate emits mp3, NOTE Chromecast devices appear to  not care if incorrect
    base_url = item.config.get('baseuri', os.environ.get('GOOGLE_TRANSLATE_URL', 'https://translate.google.com/translate_tts?'))
    lang = item.config.get('lang', 'en')  # English

    # Generate a Google Translate (compatible) URL to generate TTS
    vars = {
        'q': message,
        'l': lang,
        'tl': lang,
        'client': 'tw-ob',
        'ttsspeed': 1,
        'total': 1,
        'ie': 'UTF-8',
        # looks like can get away with out 'textlen'
    }
    url = base_url + urlencode(vars)


    # TODO disable pychromecast library logging?
    import pychromecast  # Some plugin systems want lazy loading, defer import until about to use it

    chromecasts, browser  = pychromecast.get_listed_chromecasts(friendly_names=item.addrs)
    if not chromecasts:
        return False
    for cast in chromecasts:
        cast.wait()
        #srv.logging.debug("cast=%r", cast)
        """
        print('%r' % (cast.device.friendly_name,))
        print('%r' % (cast.socket_client.host,))
        """
        mc = cast.media_controller
        mc.play_media(url, content_type=mime_type)
        mc.block_until_active()
        mc.play()  # issue play, return immediately
        # TODO detect when play failed
        # if one end point works, but another fails is that a success or a failure?
        # What is an end point does NOT show up in list? i.e. len(item.addrs) > len(chromecasts)

    return True




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from builtins import str

__author__ = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2017 Jan-Piet Mens'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import os
import sys
from mastodon import Mastodon


def plugin(srv, item):
    _DEFAULT_URL = 'https://mastodon.social'

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    # item.config is brought in from the configuration file
    config = item.config

    clientcreds, usercreds, base_url = item.addrs

    text = item.message

    try:
        mastodon = Mastodon(
            client_id=clientcreds,
            access_token=usercreds,
            api_base_url=base_url
        )

        mastodon.toot(text)
    except Exception as e:
        srv.logging.warning("Cannot post to Mastodon: %s" % (str(e)))
        return False

    return True


if __name__ == '__main__':
    from mastodon import Mastodon

    try:
        base_url, email, password, clientname, clientcred, usercred = sys.argv[1:]
    except:
        print("Usage: %s base_url email password clientname clientcredsfile usercredsfile" % sys.argv[0])
        sys.exit(2)

    if not os.path.isfile(clientcred):
        Mastodon.create_app(
            clientname,
            to_file=clientcred,
            scopes=['read', 'write'],
            api_base_url=base_url)

    if not os.path.isfile(usercred):
        mastodon_api = Mastodon(
            client_id=clientcred,
            api_base_url=base_url)

        mastodon_api.log_in(
            email,
            password,
            to_file=usercred,
            scopes=['read', 'write'],
        )



# -*- coding: utf-8 -*-
# (c) 2014-2021 The mqttwarn developers
import os
import platform

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

requires = [
    'six>=1.13.0',
    'paho-mqtt>=1.4.0',
    'jinja2>=2.10.1',
    'attrs>=19.3.0',
    'docopt>=0.6.2',
    'requests>=2.22.0',
    'future>=0.18.2',
    'configparser>=3.5.3',
]

extras = {
    'amqp': [
        'puka>=0.0.7',
    ],
    'apns': [
        'apns>=2.0.1',
    ],
    'apprise': [
        'apprise>=0.8.9',
    ],
    'asterisk': [
        'pyst2>=0.5.0',
    ],
    'celery': [
        'celery',
    ],
    'chromecast': [
        'pychromecast>=7.5.0',
    ],
    'dnsupdate': [
        'dnspython>=1.15.0',
    ],
    'fbchat': [
        'fbchat>=1.3.6',
    ],
    'gss': [
        'gdata>=2.0.18',
    ],
    'gss2': [
        'gspread>=2.1.1',
        'oauth2client>=4.1.2',
        #'google-api-python-client>=1.7.11',
    ],
    'mysql': [
        'mysql',
    ],
    'nma': [
        'PyNMA>=1.0',
    ],
    'nsca': [
        'pynsca>=1.6',
    ],
    'osxnotify': [
        'pync>=1.6.1',
    ],
    'pastebinpub': [
        'Pastebin>=1.1.2',
    ],
    'postgres': [
        'psycopg2-binary>=2.7.4',
    ],
    'prowl': [
        'pyprowl>=3.0.1',
    ],
    'pushbullet': [
        'PushbulletPythonLibrary>=2.3',
    ],
    'redispub': [
        'redis>=2.10.6',
    ],
    'rrdtool': [
        'rrdtool>=0.1.12',
    ],
    'serial': [
        'pyserial>=3.4',
    ],
    'slack': [
        'slack-sdk>=3.1.0',
    ],
    'ssh': [
        'paramiko>=2.4.1',
    ],
    'tootpaste': [
        'Mastodon.py>=1.2.2',
    ],
    'twilio': [
        'twilio>=6.11.0',
    ],
    'twitter': [
        'python-twitter>=3.4.1',
    ],
    'websocket': [
        'websocket-client>=0.47.0',
    ],
    'xively': [
        'xively-python',
    ],
    'xmpp': [
        'xmpppy>=0.6.1',
        'dnspython>=1.16.0',
    ],
    'slixmpp': [
        'slixmpp>=1.5.2',
    ],
}


# Convenience extra to install *all* dependencies for building a `mqttwarn-full` distribution.
extras_all = []
for extra, packages in extras.items():

    # FIXME: Skip specific packages having build issues.
    # https://github.com/commx/python-rrdtool/issues/36
    if extra in ["mysql", "rrdtool"]:
        continue

    # FIXME: Skip specific packages on specific platforms,
    #        because they would also need a build toolchain.
    machine = platform.uname()[4]
    if machine in ["armv7l", "aarch64"] and extra in ["postgres", "slixmpp", "ssh"]:
        continue

    # FIXME: The `cryptography` package is not available as binary wheel on arm32v7.
    if machine in ["armv7l"] and extra in ["apprise"]:
        continue

    for package in packages:
        extras_all.append(package)

extras["all"] = extras_all


# Packages needed for running the tests.
extras["test"] = [
    'pytest>=4.6.7',
    'pytest-cov>=2.8.1',
    'lovely.testlayers>=0.7.1',
    'tox>=3.14.2',
    'surrogate==0.1',
    'dataclasses; python_version<"3.7"',
    'requests-toolbelt>=0.9.1,<1',
    'responses>=0.13.3,<1',
]


setup(name='mqttwarn',
      version='0.25.0',
      description='mqttwarn - subscribe to MQTT topics and notify pluggable services',
      long_description=README,
      license="EPL 2.0",
      classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: Plugins",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Manufacturing",
        "Intended Audience :: Science/Research",
        "Intended Audience :: System Administrators",
        "Intended Audience :: Telecommunications Industry",
        "License :: OSI Approved :: Eclipse Public License 2.0 (EPL-2.0)",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Operating System :: MacOS",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Communications",
        "Topic :: Education",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: XMPP",
        "Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator",
        "Topic :: Software Development :: Embedded Systems",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Pre-processors",
        "Topic :: Software Development :: Testing",
        "Topic :: System :: Archiving",
        "Topic :: System :: Distributed Computing",
        "Topic :: System :: Monitoring",
        "Topic :: System :: Networking :: Monitoring",
        "Topic :: System :: Systems Administration",
        "Topic :: Text Processing",
        "Topic :: Utilities",
      ],
      author='Jan-Piet Mens, Ben Jones, Andreas Motl',
      author_email='jpmens@gmail.com',
      url='https://github.com/jpmens/mqttwarn',
      keywords='mqtt notification plugins data acquisition push transformation engine ' +
               'mosquitto ',
      packages=find_packages(),
      include_package_data=True,
      package_data={
        'mqttwarn': [
          '*.ini',
        ],
      },
      zip_safe=False,
      test_suite='tests',
      install_requires=requires,
      extras_require=extras,
      tests_require=extras['test'],
      entry_points={
          'console_scripts': [
              'mqttwarn = mqttwarn.commands:run',
          ],
      },
)
